local LDB = LibStub("LibDataBroker-1.1", true)
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale("WTFrames")
local WTFrames = LibStub("AceAddon-3.0"):NewAddon("WTFrames", "AceConsole-3.0")

local backdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		insets = {left = -1.5, right = -1.5, top = -1.5, bottom = -1.5},
}	
local statusBarTexture = [[Interface\Addons\WTFrames\media\statusbar]]

local anchorRef = {}
local initLDB = {}
local InitConfig = {}

function InitConfig()
	WTFramesConfig = {
		options = {
			ShowRaidIcons = true,
			FrameWidth = 120,
			FrameHeight = 20,
			HealthHeight = 15,
			IconSize = 16,
		},
		frame = {
			point = "BOTTOMLEFT",
			x = 50,
			y = 500,
		}
	}	
end

oUF.TagEvents["[exthp]"] = "UNIT_HEALTH"
oUF.Tags["[exthp]"] = function(unit)
	return UnitIsDead(unit) and "Dead" 
	    or UnitIsGhost(unit) and "Ghost" 
	    or not UnitIsConnected(unit) and "Offline" 
		or string.format('%d%%', floor(UnitHealth(unit) / UnitHealthMax(unit) * 100))
end

oUF.TagEvents["[truncname]"] = "UNIT_TARGET"
oUF.Tags["[truncname]"] = function(unit)
	local name = UnitName(unit)
	if (string.len(name) > 10) then
		name = string.format("%s...", name:sub(1, 10))
	end
	
	return name
end

local function chatCommandCallback(input) 	
	local cmd = WTFrames:GetArgs(input)
	
	if(cmd == "lock")then
		WTFrames:Lock()
	elseif(cmd == "unlock")then
		WTFrames:UnLock()
	elseif(cmd == "reset")then
		WTFrames:Reset()		
	else
		WTFrames:Print(string.format("Who's tanking - Frames (Version %s):", GetAddOnMetadata("WTFrames", "Version") or "N/A"))
		WTFrames:Print("/wtframes lock")
		WTFrames:Print("/wtframes unlock")
	end
end


function WTFrames:OnInitialize()
	WTFrames:UpdateConfig()
	
	initLDB()
	--LibStub("AceConfig-3.0"):RegisterOptionsTable(L["WTFrames"], options)
	--LibStub("AceConfigDialog-3.0"):AddToBlizOptions(L["WTFrames"])
	
	WTFrames:RegisterChatCommand("wtframes", chatCommandCallback)
	
end
 
function WTFrames:UpdateConfig()
	if not WTFramesConfig then
		InitConfig()
	end

	-- TODO
end
 
function initLDB()
	if LDB then
		local launcher = LibStub("LibDataBroker-1.1"):NewDataObject("WTFrames", {
			type = "launcher",
			label = "WTFrames",
			icon = "Interface\\Icons\\Ability_Warrior_DefensiveStance",
			OnClick = function(self, button)
				if button == "LeftButton" then
					--if InCombatLockdown() then return end
					if  anchorRef:IsVisible() then
						WTFrames:Lock()
					else
						WTFrames:UnLock()
					end
				end
			end,
			OnTooltipShow = function(tooltip)
				if not tooltip or not tooltip.AddLine then return end
				tooltip:AddLine("Who's tanking - Frames")
				tooltip:AddLine("|cffffff00" .. L["Left-click to toggle anchor."] .. "|r")
				--tooltip:AddLine("|cffffff00" .. L["Right-click for options."] .. "|r")
			end
		})		
	end
end

function WTFrames:Lock()
	if anchorRef:IsVisible() then
		WTFrames:Print(L["Maintank frames are now locked"])
		anchorRef:Hide()
	end
end

function WTFrames:UnLock()
	if not anchorRef:IsVisible() then
		WTFrames:Print(L["Maintank frames are now unlocked"])
		anchorRef:Show()
	end
end

function WTFrames:Reset()	
	InitConfig()
end


function oRA3MaintankStyle(self, unit)

	local frameheight = WTFramesConfig.options.FrameHeight
	local framewidth = WTFramesConfig.options.FrameWidth
	local healthheight = WTFramesConfig.options.HealthHeight	

    self:SetAttribute('initial-height', frameheight)
    self:SetAttribute('initial-width', framewidth)      
	
	local hp = CreateFrame('StatusBar', "HealthBar", self) 
    hp:SetPoint("TOPLEFT") 
    hp:SetStatusBarTexture(statusBarTexture) 
	hp:SetWidth(framewidth)
    hp:SetHeight(healthheight) 
    hp:SetPoint("RIGHT") 
    hp:SetPoint("LEFT")
	
    hp.bg = hp:CreateTexture(nil, "BORDER")
    hp.bg:SetAllPoints(hp)
    hp.bg:SetTexture(statusBarTexture)
   	hp.bg.multiplier = 0.3
	
	hp.colorClass = true
	hp.colorClassNPC = true
 	hp.colorDisconnected = true;
 	hp.frequentUpdates = true;
	
	local textHP = hp:CreateFontString(nil, 'OVERLAY')
	textHP:SetFontObject(GameFontNormalSmall)
	textHP:SetShadowColor(0,0,0)
	textHP:SetShadowOffset(0.8, -0.8)
	textHP:SetTextColor(1,1,1)
	textHP:SetJustifyH("LEFT")	
	textHP:SetPoint('RIGHT',self,'RIGHT',-2,2)
	textHP.frequentUpdates = true	   	
    self:Tag(textHP,'[exthp]')				
			
	local textName = hp:CreateFontString(nil, 'OVERLAY')
	textName:SetFontObject(GameFontNormalSmall)
	textName:SetShadowColor(0,0,0)
	textName:SetShadowOffset(0.8, -0.8)
	textName:SetTextColor(1,1,1)
	textName:SetJustifyH("LEFT")	
	textName:SetPoint('LEFT',self,'LEFT',17,2)
	textName.frequentUpdates = true	   	
    self:Tag(textName,'[truncname]')
	
	
	local pp = CreateFrame('StatusBar', nil, self)
	pp:SetHeight(frameheight - healthheight - 1)
	pp:SetStatusBarTexture(statusBarTexture)
	--pp:SetParent(self) 
	pp:SetPoint("LEFT")
	pp:SetPoint("RIGHT")
	pp:SetPoint("TOP", hp, "BOTTOM", 0, 0)
	pp.colorPower = true
	pp.frequentUpdates = true
	pp.bg = pp:CreateTexture(nil, "BORDER")
	pp.bg:SetAllPoints(pp)
	pp.bg:SetTexture(statusbartexture)
	pp.bg:SetAlpha(.5)	

	self.Power = pp
	
		  
    if(WTFramesConfig.options.ShowRaidIcons)then
        local raidIcon = hp:CreateTexture(nil, "OVERLAY")
        raidIcon:SetPoint("LEFT", self, 0, 2)
        raidIcon:SetHeight(WTFramesConfig.options.IconSize)
        raidIcon:SetWidth(WTFramesConfig.options.IconSize)
        raidIcon.frequentUpdates = 0.1
		
		self.RaidIcon = raidIcon
    end
	
	self.Health = hp   
	
 end
 

 
function WTFrames:OnEnable()	
	oUF:RegisterStyle('oRA3MaintankStyle', oRA3MaintankStyle) 
	oUF:SetActiveStyle("oRA3MaintankStyle")
	
	local oRA3Tanks = oRA3:GetSortedTanks()
	
	local anchor = CreateFrame("Frame", nil, UIParent)
	local anchorText = anchor:CreateFontString(nil, nil, "GameFontNormalSmall")
	anchorText:SetPoint("TOPLEFT")
	anchorText:SetText("WTFrames Anchor")
	anchor:SetWidth(WTFramesConfig.options.FrameWidth)
	--anchor:SetHeight(24 + (#oRA3Tanks * (WTFramesConfig.options.FrameHeight + 2.9)))
	anchor:SetHeight(12)

	--anchor:SetBackdrop(backdrop)
	--anchor:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
	--anchor:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)		
	
	anchor:SetPoint(WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y )
	
	anchor:EnableMouse(true)
	anchor:SetMovable(true)
	anchor:SetScript("OnMouseDown", anchor.StartMoving)
	anchor:SetScript("OnMouseUp", function(self, button)
		self:StopMovingOrSizing()
		WTFramesConfig.frame.point, WTFramesConfig.frame.x, WTFramesConfig.frame.y = "BOTTOMLEFT", self:GetLeft(), self:GetBottom()
	end)
	
	anchor:Hide()
			
	anchorRef = anchor
	
	local maintanks = oUF:Spawn("header", "WTFrames_MainTanks")
	--maintanks:SetPoint("TOPLEFT", 'UIParent', "TOPLEFT", 15, -500)
	maintanks:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, 0)
	maintanks:SetManyAttributes(
		"yOffset", 2.9,
		"nameList", table.concat(oRA3Tanks, ","),
		"template", "WTFrames_Maintank",
		"showRaid", true,
		"initial-unitWatch", true,
		"point", "BOTTOM",
		"sortDir", "DESC"
	)
	local tankhandler = {}
	function tankhandler:OnTanksUpdated(event, tanks) 
		maintanks:SetAttribute("nameList", table.concat(tanks, ","))			
		--anchor:SetHeight(24 + (#tanks * (WTFramesConfig.options.FrameHeight + 2.9)))
	end
	oRA3.RegisterCallback(tankhandler, "OnTanksUpdated")
			
	maintanks:Show()
end


